from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import File, Int, String, Boolean

Verse_V0_1_0 = CommandToolBuilder(tool="verse", base_command=["verse"], inputs=[ToolInput(tag="in_give_name_onlysupports", input_type=File(optional=True), prefix="-a", doc=InputDocumentation(doc="Give the name of the annotation file. The program currently only\nsupports GTF format.")), ToolInput(tag="in_give_general_name", input_type=File(optional=True), prefix="-o", doc=InputDocumentation(doc="Give the general name of the output file, e.g., 'Sample_A'.\nThe summary file will be named 'Sample_A.summary.txt.'\nThe file containing gene counts will be named 'Sample_A.exon.txt',\n'Sample_A.intron.txt', etc.")), ToolInput(tag="in_running_mode_htseqintersectionnonempty", input_type=Int(optional=True), prefix="-z", doc=InputDocumentation(doc="The Running Mode: 0 by default (featureCounts), 1 (HTSeq-Union),\n2 (HTSeq-Intersection_strict), 3 (HTSeq-Intersection_nonempty),\n4 (Union_strict), 5 (Cover_length).\nPlease refer to the manual or use '-Z' to check the details of\neach mode.")), ToolInput(tag="in_specify_feature_type", input_type=File(optional=True), prefix="-t", doc=InputDocumentation(doc="Specify the feature type. Only rows which have the matched\nfeature type in the provided GTF annotation file will be included\nfor read counting. 'exon' by default.\nProviding a list of feature types (e.g., -t 'exon;intron;mito')\nwill automatically enter hierarchical_assign mode. If the user\nwants to assign independently for each feature type, please\nspecify '--assignIndependently'. Use -Z to check the details.")), ToolInput(tag="in_specify_geneidentifier_attribute", input_type=String(optional=True), prefix="-g", doc=InputDocumentation(doc="Specify the gene_identifier attribute, which is normally 'gene_id'\nor 'gene_name'. 'gene_id' by default.")), ToolInput(tag="in_input_file_pairedend", input_type=Boolean(optional=True), prefix="-S", doc=InputDocumentation(doc="If the input file is paired-end data but not sorted by read name,\nthis option MUST be specified.")), ToolInput(tag="in_indicate_strandspecific_read", input_type=Int(optional=True), prefix="-s", doc=InputDocumentation(doc="Indicate if strand-specific read counting should be performed.\nIt has three possible values:  0 (unstranded), 1 (stranded) and\n2 (reversely stranded). 0 by default.")), ToolInput(tag="in_minimum_mapping_quality", input_type=Int(optional=True), prefix="-Q", doc=InputDocumentation(doc="The minimum mapping quality score a read must satisfy in order\nto be counted. For paired-end reads, at least one end should\nsatisfy this criteria. 0 by default.")), ToolInput(tag="in_output_gene_length", input_type=Boolean(optional=True), prefix="-l", doc=InputDocumentation(doc="Output the gene length. This length is feature_type specific,\nwhich means the length in an exon_count file will be the total\nexon length of each gene, the length in an intron_count file will\nbe the total intron length, etc.")), ToolInput(tag="in_output_read_details", input_type=File(optional=True), prefix="-R", doc=InputDocumentation(doc="Output read assignment details for each read/read pairs. The\ndetails will be saved to a tab-delimited file that contains five\ncolumns including read name, status(assigned or the reason if not\nassigned), feature type and assigned gene name.")), ToolInput(tag="in_number_threads_used", input_type=Int(optional=True), prefix="-T", doc=InputDocumentation(doc="Number of threads used for read assignment. 1 by default.\nNote that when running, VERSE will initiate one main thread and\nat least one helper thread for read assignment. This option\nspecifies the number of helper threads.")), ToolInput(tag="in_single_end", input_type=Boolean(optional=True), prefix="--singleEnd", doc=InputDocumentation(doc="If specified, VERSE will assume the input is\nsingle-end data, and assign every reads individually. If this is\nnot specified(default), the input will be treated as paired-end\ndata. The 'missing mate' count will show how many reads are not\nin pairs. VERSE can correctly assign single-end data in paired-\nend mode, but it will take longer and the reads will all be\ncounted as missing mates. So it is recommended to specify this\nif the user knows it is single-end.")), ToolInput(tag="in_assign_independently", input_type=Boolean(optional=True), prefix="--assignIndependently", doc=InputDocumentation(doc="If specified, VERSE will assign reads to listed\nfeature types independently. This has the same effect as running\nVERSE separately for each feature type, except that it only\nrequires one run, thus is more efficient.")), ToolInput(tag="in_read_extension_five", input_type=Int(optional=True), prefix="--readExtension5", doc=InputDocumentation(doc="Reads are extended upstream by <int> bases from\ntheir 5' end.")), ToolInput(tag="in_read_extension_three", input_type=Int(optional=True), prefix="--readExtension3", doc=InputDocumentation(doc="Reads are extended upstream by <int> bases from\ntheir 3' end.")), ToolInput(tag="in_min_read_overlap", input_type=Int(optional=True), prefix="--minReadOverlap", doc=InputDocumentation(doc="Specify the minimum number of overlapped bases\nrequired to assign a read to a feature. 1 by default.")), ToolInput(tag="in_max_read_non_overlap", input_type=Int(optional=True), prefix="--maxReadNonoverlap", doc=InputDocumentation(doc="Specify the maximum number of non-overlapped bases\na read can have. A read is considered no_feature if its number of\nnon-overlapped bases exceeds this threshold. This option is not\nuseful in strict mode since it requires the assigned feature to\noverlap every base of the read.")), ToolInput(tag="in_count_split_alignments_only", input_type=Boolean(optional=True), prefix="--countSplitAlignmentsOnly", doc=InputDocumentation(doc="If specified, only split alignments (CIGAR\nstrings containing letter `N') will be counted. All the other\nalignments will be ignored. An example of split alignments is\nthe exon-spanning reads in RNA-seq data.")), ToolInput(tag="in_read_two_pos", input_type=Boolean(optional=True), prefix="--read2pos", doc=InputDocumentation(doc="<5:3>            The read is reduced to its 5' most base or 3'\nmost base. Read summarization is then performed based on the\nsingle base which the read is reduced to.")), ToolInput(tag="in_ignored_up", input_type=Boolean(optional=True), prefix="--ignoreDup", doc=InputDocumentation(doc="If specified, reads that were marked as\nduplicates will be ignored. Bit Ox400 in FLAG field of SAM/BAM\nfile is used for identifying duplicate reads. In paired end\ndata, the entire read pair will be ignored if at least one end\nis found to be a duplicate read.")), ToolInput(tag="in_mind_if_ambiguous", input_type=Int(optional=True), prefix="--minDifAmbiguous", doc=InputDocumentation(doc="This option can only be used in VERSE-cover_length\nmode. When the read or the read pair hits more than one genes,\nthe number of bases overlapped by each gene will be calculated.\nIf the covered_length difference between the most covered gene\nand the second most covered gene is smaller than this specified\nvalue, the read will be set ambiguous. 1 base difference by default.")), ToolInput(tag="in_non_empty_modified", input_type=Boolean(optional=True), prefix="--nonemptyModified", doc=InputDocumentation(doc="This option can only be used in intersection_\nnonempty mode. In cases where HTSeq would assign multi-hit reads\nto no_feature, VERSE will assign those to ambiguous.")), ToolInput(tag="in_multithread_decompress", input_type=Boolean(optional=True), prefix="--multithreadDecompress", doc=InputDocumentation(doc="BAM files generated with SAMTools or STAR after\nyear 2013 has a slight format change which allows multithreaded\ndecompression. BAM processing will be faster if this option is\nspecifed and multiple core is used.")), ToolInput(tag="in_specified_length_tlen", input_type=Boolean(optional=True), prefix="-P", doc=InputDocumentation(doc="If specified, template length (TLEN in SAM specification) will be\nchecked when assigning read pairs (templates) to genes. This option\nis only applicable in paired-end mode. The distance thresholds\nshould be specified using -d and -D options.")), ToolInput(tag="in_minimum_templateread_pair", input_type=Int(optional=True), prefix="-d", doc=InputDocumentation(doc="Minimum template(read pair) length, 50 by default.")), ToolInput(tag="in_maximum_templateread_pair", input_type=Int(optional=True), prefix="-D", doc=InputDocumentation(doc="Maximum template(read pair) length, 600 by default.")), ToolInput(tag="in_specified_only_read", input_type=Boolean(optional=True), prefix="-B", doc=InputDocumentation(doc="If specified, only read pairs that have both ends successfully\naligned will be considered for summarization. This option is only\napplicable for paired-end reads.")), ToolInput(tag="in_specified_chimeric_read", input_type=Boolean(optional=True), prefix="-C", doc=InputDocumentation(doc="If specified, the chimeric read pairs (those that have their two\nends aligned to different chromosomes) will NOT be included for\nsummarization. This option is only applicable for paired-end data.")), ToolInput(tag="in_output_version_program", input_type=Boolean(optional=True), prefix="-v", doc=InputDocumentation(doc="Output version of the program.")), ToolInput(tag="in_show_details_running", input_type=Boolean(optional=True), prefix="-Z", doc=InputDocumentation(doc="Show details about the running mode or scheme.")), ToolInput(tag="in_input_file", input_type=String(), position=0, doc=InputDocumentation(doc="Give the name of input read file that contains the read mapping\nresults. Format of input file is automatically determined (SAM/BAM)\nIf it is paired-end data, the file MUST be name-sorted, otherwise\nthe user MUST specify -S, or use samtools to sort it by name."))], outputs=[ToolOutput(tag="out_give_general_name", output_type=File(optional=True), selector=InputSelector(input_to_select="in_give_general_name", type_hint=File()), doc=OutputDocumentation(doc="Give the general name of the output file, e.g., 'Sample_A'.\nThe summary file will be named 'Sample_A.summary.txt.'\nThe file containing gene counts will be named 'Sample_A.exon.txt',\n'Sample_A.intron.txt', etc.")), ToolOutput(tag="out_output_read_details", output_type=File(optional=True), selector=InputSelector(input_to_select="in_output_read_details", type_hint=File()), doc=OutputDocumentation(doc="Output read assignment details for each read/read pairs. The\ndetails will be saved to a tab-delimited file that contains five\ncolumns including read name, status(assigned or the reason if not\nassigned), feature type and assigned gene name."))], container=None, version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Verse_V0_1_0().translate("wdl", allow_empty_container=True)

